
%{

/**
@todo Recognize comments after statements like:
SOURCES = foo #regognize me

@fixme Parser fails on files that do not end with a newline
@fixme 1 shift/reduce conflict in "line_body" rule
*/

#include <qvaluestack.h>
#include "qmakeast.h"

#define YYSTYPE_IS_DECLARED

using namespace QMake;

struct Result {
    Result(): node(0) {}

    QString value;
    AST *node;
    QStringList values;
};

typedef Result YYSTYPE;

void yyerror(const char *str) {
    printf("%s", str);
}

int yylex();

QValueStack<ProjectAST*> projects;

int depth = 0;

/*
%debug
*/
%}

%token ID_SIMPLE
%token ID_LIST
%token LBRACE
%token EQ
%token PLUSEQ
%token MINUSQE
%token STAREQ
%token TILDEEQ
%token LBRACE
%token RBRACE
%token NOT
%token COLON
%token NUMSIGN
%token NEWLINE
%token NUMBER
%token COMMENT
%token CONT
%token DOT
%token ID_EX
%token RCURLY
%token LCURLY
%token ID_ARGS
%token LIST_COMMENT
%token ID_LIST_SINGLE

%%

project : 
    {
        ProjectAST *projectAST = new ProjectAST();
        projects.push(projectAST);
    }
    statements
    ;
    
statements : statements statement
        {
            projects.top()->addChildAST($<node>2);
            $<node>2->setDepth(depth);
        }
    |
    ;

statement : variable_assignment 
        {
            $<node>$ = $<node>1;
        }
    | scope 
        {
            $<node>$ = $<node>1;
        }
    | function_call
        {
            $<node>$ = $<node>1;
        }
    | comment
        {
            $<node>$ = $<node>1;
        }
    | emptyline
        {
            $<node>$ = new NewLineAST();
        }
    ;

variable_assignment : scoped_identifier operator multiline_values
        {
            AssignmentAST *node = new AssignmentAST();
            node->scopedID = $<value>1;
            node->op = $<value>2;
            node->values = $<values>3;
            $<node>$ = node;
        }
    ;

scoped_identifier : ID_SIMPLE COLON scoped_identifier  
        { $<value>$ = $<value>1 + $<value>2 + $<value>3; }
    | ID_SIMPLE COLON  { $<value>$ = $<value>1 + $<value>2; }
    | ID_SIMPLE
    ;

multiline_values : multiline_values line_body
        {
            $<values>$.append($<value>2);
        }
    |   { $<values>$.clear(); }
    ;

line_body : ID_LIST CONT         { $<value>$ = $<value>1 + " \\\n"; }
    | ID_LIST_SINGLE NEWLINE     { $<value>$ = $<value>1 + "\n"; }
    | CONT                       { $<value>$ = "\\\n"; }
    | LIST_COMMENT
    | RBRACE                     { $<value>$ = ""; }
    ;


operator : EQ | PLUSEQ | MINUSQE | STAREQ | TILDEEQ
    ;
    
scope : scoped_identifier 
        {
            ProjectAST *projectAST = new ProjectAST(ProjectAST::Scope);
            projects.push(projectAST);
            projects.top()->scopedID = $<value>1;
            depth++;
        }
    scope_body
        {
            $<node>$ = projects.pop();
            depth--;
        }
    ;

function_call : scoped_identifier LBRACE function_args RBRACE 
        {
            ProjectAST *projectAST = new ProjectAST(ProjectAST::FunctionScope);
            projects.push(projectAST);
            projects.top()->scopedID = $<value>1;
            projects.top()->args = $<value>3;
            depth++;
        }
    scope_body 
    else_statement
        {
            $<node>$ = projects.pop();
            depth--;
        }
    | scoped_identifier LBRACE function_args RBRACE COLON variable_assignment
        {
            FunctionCallAST *node = new FunctionCallAST();
            node->scopedID = $<value>1;
            node->args = $<value>3;
            node->assignment = static_cast<AssignmentAST*>($<node>6);
            $<node>$ = node;
        }
    ;
    
function_args : ID_ARGS    { $<value>$ = $<value>1; }
    |    { $<value>$ = ""; }
    ;

scope_body : LCURLY statements RCURLY
    |
    ;
    
else_statement : "else" LCURLY 
        {
            ProjectAST *projectAST = new ProjectAST(ProjectAST::FunctionScope);
            projects.push(projectAST);
            projects.top()->scopedID = "else";
            projects.top()->args = "";
            depth++;
        }
    statements RCURLY
        {
            $<node>$ = projects.pop();
            depth--;
        }
    | "else" COLON variable_assignment
        {
            FunctionCallAST *node = new FunctionCallAST();
            node->scopedID = "else";
            node->args = "";
            node->assignment = static_cast<AssignmentAST*>($<node>3);
            $<node>$ = node;
        }
    |
        {
            $<node>$ = new FunctionCallAST();
        }
    ;

comment : COMMENT NEWLINE
        {
            CommentAST *node = new CommentAST();
            node->comment = $<value>1 + "\n";
            $<node>$ = node;
        }
    ;

emptyline : NEWLINE
    ;
    
%%

#include "qmake_lex.cpp"
