/***************************************************************************
 *   Copyright (C) 2005 by Alexander Dymo                                  *
 *   adymo@kdevelop.org                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef QMAKEQMAKEAST_H
#define QMAKEQMAKEAST_H

#include <qstringlist.h>

namespace QMake {

class AST {
public:
    enum NodeType {
        ProjectAST,
        AssignmentAST,
        NewLineAST,
        CommentAST,
        FunctionCallAST
    };
    
    AST(NodeType nodeType): m_nodeType(nodeType), m_depth(0) {}
    virtual ~AST();
    
    virtual void addChildAST(AST *node);
    virtual void writeBack(QString &buffer);
    
    virtual NodeType nodeType() const { return m_nodeType; }
    
    void setDepth(int depth) { m_depth = depth; }
    int depth() const { return m_depth; }
    virtual QString indentation();
    
protected:
    NodeType m_nodeType;
    QValueList<AST*> m_children;
    
private:
    int m_depth;

};

class StatementAST: public AST {
public:
    StatementAST(NodeType nodeType): AST(nodeType) {}

};

class ProjectAST: public StatementAST {
public:
    enum Kind { Project, Scope, FunctionScope };
    
    ProjectAST(Kind kind = Project): StatementAST(AST::ProjectAST), m_kind(kind) {}
    
    virtual void writeBack(QString &buffer);
    virtual void addChildAST(AST *node);

    bool isProject() const { return m_kind == Project; }
    bool isScope() const { return m_kind == Scope; }
    bool isFunctionScope() const { return m_kind == FunctionScope; }
    
    QString scopedID;
    QString args;
    QValueList<QMake::StatementAST*> statements;
    
private:
    Kind m_kind;

};

class AssignmentAST: public StatementAST {
public:
    AssignmentAST(): StatementAST(AST::AssignmentAST) {}

    virtual void writeBack(QString &buffer);

    QString scopedID;
    QString op;
    QStringList values;
};

class NewLineAST: public StatementAST {
public:
    NewLineAST(): StatementAST(AST::NewLineAST) {}
    
    virtual void writeBack(QString &buffer);

};

class CommentAST: public StatementAST {
public:
    CommentAST(): StatementAST(AST::CommentAST) {}
    
    virtual void writeBack(QString &buffer);
    
    QString comment;

};

class FunctionCallAST: public StatementAST {
public:
    FunctionCallAST(): StatementAST(AST::FunctionCallAST), assignment(0) {}
    
    virtual void writeBack(QString &buffer);
    
    QMake::AssignmentAST *assignment;
    QString scopedID;
    QString args;
};

}

#endif
